@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2014 Adrian Thurston <thurston@colm.net>
+ * Copyright 2006-2018 Adrian Thurston <thurston@colm.net>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -163,6 +163,7 @@ void BaseParser::init()
 
 	/* Internal variables. */
 	addArgvList();
+	addStdsList();
 }
 
 void BaseParser::addRegularDef( const InputLoc &loc, Namespace *nspace,
@@ -448,6 +449,13 @@ void BaseParser::addArgvList()
 	pd->argvTypeRef = TypeRef::cons( internal, TypeRef::List, 0, elType, valType );
 }
 
+void BaseParser::addStdsList()
+{
+	TypeRef *valType = TypeRef::cons( internal, pd->uniqueTypeStream );
+	TypeRef *elType = TypeRef::cons( internal, TypeRef::ListEl, valType );
+	pd->stdsTypeRef = TypeRef::cons( internal, TypeRef::List, 0, elType, valType );
+}
+
 ObjectDef *BaseParser::blockOpen()
 {
 	/* Init the object representing the local frame. */
@@ -691,7 +699,7 @@ LangExpr *BaseParser::send( const InputLoc &loc, LangVarRef *varRef,
 		ConsItemList *list, bool eof )
 {
 	ParserText *parserText = ParserText::cons( loc,
-			curNspace(), list, true, false, "" );
+			curNspace(), list, true, false, false, "" );
 	pd->parserTextList.append( parserText );
 
 	return LangExpr::cons( LangTerm::consSend( loc, varRef,
@@ -702,7 +710,7 @@ LangExpr *BaseParser::sendTree( const InputLoc &loc, LangVarRef *varRef,
 		ConsItemList *list, bool eof )
 {
 	ParserText *parserText = ParserText::cons( loc,
-			curNspace(), list, true, false, "" );
+			curNspace(), list, true, false, false, "" );
 	pd->parserTextList.append( parserText );
 
 	return LangExpr::cons( LangTerm::consSendTree( loc, varRef,
@@ -711,7 +719,7 @@ LangExpr *BaseParser::sendTree( const InputLoc &loc, LangVarRef *varRef,
 
 LangExpr *BaseParser::parseCmd( const InputLoc &loc, bool tree, bool stop,
 		ObjectField *objField, TypeRef *typeRef, FieldInitVect *fieldInitVect,
-		ConsItemList *list, bool used, bool reduce, const String &reducer )
+		ConsItemList *list, bool used, bool reduce, bool read, const String &reducer )
 {
 	LangExpr *expr = 0;
 
@@ -733,7 +741,7 @@ LangExpr *BaseParser::parseCmd( const InputLoc &loc, bool tree, bool stop,
 		used = true;
 
 	ParserText *parserText = ParserText::cons( loc, curNspace(),
-			list, used, reduce, reducer );
+			list, used, reduce, read, reducer );
 	pd->parserTextList.append( parserText );
 
 	LangTerm::Type langTermType = stop ? LangTerm::ParseStopType : ( tree ? 
@@ -957,8 +965,17 @@ LangExpr *BaseParser::match( const InputLoc &loc, LangVarRef *varRef,
 			list, pd->nextPatConsId++ );
 	pd->patternList.append( pattern );
 
-	LangExpr *expr = LangExpr::cons( LangTerm::cons( 
-			InputLoc(), LangTerm::MatchType, varRef, pattern ) );
+	LangExpr *expr = LangExpr::cons( LangTerm::consMatch( 
+			InputLoc(), varRef, pattern ) );
+
+	return expr;
+}
+
+LangExpr *BaseParser::prodCompare( const InputLoc &loc, LangVarRef *varRef,
+		const String &prod, LangExpr *matchExpr )
+{
+	LangExpr *expr = LangExpr::cons( LangTerm::consProdCompare( 
+			InputLoc(), varRef, prod, matchExpr ) );
 
 	return expr;
 }
@@ -996,8 +1013,8 @@ LangExpr *BaseParser::require( const InputLoc &loc,
 			list, pd->nextPatConsId++ );
 	pd->patternList.append( pattern );
 
-	LangExpr *expr = LangExpr::cons( LangTerm::cons(
-			InputLoc(), LangTerm::MatchType, varRef, pattern ) );
+	LangExpr *expr = LangExpr::cons( LangTerm::consMatch(
+			InputLoc(), varRef, pattern ) );
 	return expr;
 }
 
